{{
HUB75 write helper object configuration

Author:         Tom Dinger (Ding-Batty)
Release Date:   2024/02/26

Copyright 2024 Thomas E. Dinger

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

--------------------------------------------------------------------------

This file contains the configuration constants for helper object and cog code
for writing into the display buffer used by the the HUB75 display object for
one or more HUB75-based display panels. It contains the command message layout
used in the command FiFo (a set of mailboxes) for sending write requests to the
helper cog.
}}

CON
  {
    Commands.
    The command values are non-zero, and fit in a 16-bit WORD.
    The first WORD of the command is the command code.
    The second WORD is an optional pointer to a (hub) variable that will
    receive an indication that the command has been completed (or failed).

    Command buffers are 32 bytes long in the FiFo (8 LONGs or 16 WORDs), so
    they can be properly aligned if they contain any LONG values (which some
    do contain).
  }

  CMD_PIXEL        = $0001
  CMD_BITMAP_1BPP  = $0002
  CMD_ROM_FONT     = $0003
  CMD_NOOP         = $0004
  ' Other commands?
  ' - write a single pixel (limit case of bitmap?)
  ' - vertical line (special case of bitmap?)
  ' - horizontal line (special case of bitmap?)
  ' - box (special case of bitmap?)
  ' - ROM Character -- kind of like a bitmap, but shifting every two bits.
  ' - line in arbitrary direction
  ' - curves/circles/ovals?

  MAX_CMD = CMD_NOOP
  MAX_CMD_PLUS_ONE = CMD_NOOP + 1

  CMD_MASK = $FF

  CMDBUF_SIZE_LONGS = 8
  CMDBUF_SIZE_WORDS = CMDBUF_SIZE_LONGS << 1
  CMDBUF_SIZE_BYTES = CMDBUF_SIZE_WORDS << 1

  {
    No-op command -- does nothing, always succeeds, does the return value
    processing.

    This command can also be used to synchronize (i.e. block) after a series
    of other commands have been enqueued in the command FiFo, and when this
    one completes, it implies that all the previous commands have also
    ompleted.
  }
  NOOPCMD_CMD            = 0
  NOOPCMD_RESULT_PTR     = 2

  {
    Pixel write command arguments
  }

  PIXELCMD_CMD           = 0
  PIXELCMD_RESULT_PTR    = 2
  PIXELCMD_COLUMN        = 4
  PIXELCMD_ROW           = 6
  PIXELCMD_COLOR         = 8  ' LONG -- 4 bytes

  {
    Bitmap command argument layout
    This uses a common prefix for a number of functions that share a large
    number of the same parameters.
  }

  BITMAPCMD_CMD            =  0   ' WORD -- 2 bytes
  BITMAPCMD_RESULT_PTR     =  2   ' WORD
  BITMAPCMD_COLUMN         =  4   ' WORD -- 2 bytes, may be negative
  BITMAPCMD_ROW            =  6   ' WORD -- 2 bytes, may be negative
  BITMAPCMD_WIDTH          =  8
  BITMAPCMD_HEIGHT         = 10
  BITMAPCMD_LTRIM          = 12   ' WORD -- 2 bytes
  BITMAPCMD_TTRIM          = 14
  ' We can have several different ways to specify what to display:
  ' - bitmap hub address + row address increment
  ' - ROM Font character
  BITMAPCMD_BITMAP_HUBADDR = 16   ' WORD -- 2 bytes
  BITMAPCMD_ROWADDR_INCR   = 18   ' WORD -- 2 bytes
  ' We can also specify the colors in a few ways:
  ' - explicit colors (each one is a LONG)
  ' - Hub address of a "palette" table (each entry is a LONG), plus the number
  '   of bits per color index (1bpp, 2bpp, etc.)
  ' It would be nice to squeeze in the Bpp value, but that might only be
  ' needed when we are using a larger color palette (bpp > 1), and so we
  ' might have some space in the 8 color bytes.
  BITMAPCMD_BGCOLOR        = 20   ' LONG -- 4 bytes
  BITMAPCMD_FGCOLOR        = 24   ' LONG -- 4 bytes
  BITMAPCMD_SKIP_PAT       = 28   ' LONG -- 4 bytes
  ' The command size is 32 BYTEs or 8 LONGs.

  {
    ROM Font character output command argument layout
    This uses a common prefix for a number of functions that share a large
    number of the same parameters.
  }

  ROMFONTCMD_CMD           =  0   ' WORD -- 2 bytes
  ROMFONTCMD_RESULT_PTR    =  2   ' WORD
  ROMFONTCMD_COLUMN        =  4   ' WORD -- 2 bytes, may be negative
  ROMFONTCMD_ROW           =  6   ' WORD -- 2 bytes, may be negative
  ROMFONTCMD_WIDTH         =  8
  ROMFONTCMD_HEIGHT        = 10
  ROMFONTCMD_LTRIM         = 12   ' WORD -- 2 bytes
  ROMFONTCMD_TTRIM         = 14
  ' Command-specific parameters
  ROMFONTCMD_ROM_CHAR      = 16
  ' The word at offset 18 is unused.
  ROMFONTCMD_BGCOLOR       = 20   ' LONG -- 4 bytes
  ROMFONTCMD_FGCOLOR       = 24   ' LONG -- 4 bytes
  ROMFONTCMD_SKIP_PAT      = 28   ' LONG -- 4 bytes
  ' The command size is 32 BYTEs or 8 LONGs.

' ========================================================================

PUB NoOp
  ' Needed for a "proper" object

